@@ -1,8 +1,7 @@
 # Copyright: (c) 2016, Allen Sanabria <asanabria@linuxdynasty.org>
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 from os import path, walk
 import re
@@ -10,8 +9,8 @@ import pathlib
 
 import ansible.constants as C
 from ansible.errors import AnsibleError
-from ansible.module_utils.six import string_types
-from ansible.module_utils.common.text.converters import to_native, to_text
+from ansible._internal._datatag._tags import SourceWasEncrypted
+from ansible.module_utils.common.text.converters import to_native
 from ansible.plugins.action import ActionBase
 from ansible.utils.vars import combine_vars
 
@@ -38,14 +37,17 @@ class ActionModule(ActionBase):
         if not self.ignore_files:
             self.ignore_files = list()
 
-        if isinstance(self.ignore_files, string_types):
+        if isinstance(self.ignore_files, str):
+            self._display.deprecated(
+                msg="Specifying 'ignore_files' as a string is deprecated.",
+                version="2.24",
+                help_text="Use a list of strings instead.",
+                obj=self.ignore_files,
+            )
             self.ignore_files = self.ignore_files.split()
 
-        elif isinstance(self.ignore_files, dict):
-            return {
-                'failed': True,
-                'message': '{0} must be a list'.format(self.ignore_files)
-            }
+        if not isinstance(self.ignore_files, list):
+            raise AnsibleError("The 'ignore_files' option must be a list.", obj=self.ignore_files)
 
     def _set_args(self):
         """ Set instance variables based on the arguments that were passed """
@@ -65,11 +67,8 @@ class ActionModule(ActionBase):
         self.ignore_files = self._task.args.get('ignore_files', None)
         self.valid_extensions = self._task.args.get('extensions', self.VALID_FILE_EXTENSIONS)
 
-        # convert/validate extensions list
-        if isinstance(self.valid_extensions, string_types):
-            self.valid_extensions = list(self.valid_extensions)
         if not isinstance(self.valid_extensions, list):
-            raise AnsibleError('Invalid type for "extensions" option, it must be a list')
+            raise AnsibleError("The 'extensions' option must be a list.", obj=self.valid_extensions)
 
     def run(self, tmp=None, task_vars=None):
         """ Load yml files recursively from a directory.
@@ -93,10 +92,10 @@ class ActionModule(ActionBase):
             elif arg in self.VALID_ALL:
                 pass
             else:
-                raise AnsibleError('{0} is not a valid option in include_vars'.format(to_native(arg)))
+                raise AnsibleError(f'{arg} is not a valid option in include_vars', obj=arg)
 
         if dirs and files:
-            raise AnsibleError("You are mixing file only and dir only arguments, these are incompatible")
+            raise AnsibleError("You are mixing file only and dir only arguments, these are incompatible", obj=self._task.args)
 
         # set internal vars from args
         self._set_args()
@@ -108,13 +107,13 @@ class ActionModule(ActionBase):
             self._set_root_dir()
             if not path.exists(self.source_dir):
                 failed = True
-                err_msg = ('{0} directory does not exist'.format(to_native(self.source_dir)))
+                err_msg = f"{self.source_dir} directory does not exist"
             elif not path.isdir(self.source_dir):
                 failed = True
-                err_msg = ('{0} is not a directory'.format(to_native(self.source_dir)))
+                err_msg = f"{self.source_dir} is not a directory"
             else:
                 for root_dir, filenames in self._traverse_dir_depth():
-                    failed, err_msg, updated_results = (self._load_files_in_dir(root_dir, filenames))
+                    failed, err_msg, updated_results = self._load_files_in_dir(root_dir, filenames)
                     if failed:
                         break
                     results.update(updated_results)
@@ -143,9 +142,8 @@ class ActionModule(ActionBase):
             result['message'] = err_msg
         elif self.hash_behaviour is not None and self.hash_behaviour != C.DEFAULT_HASH_BEHAVIOUR:
             merge_hashes = self.hash_behaviour == 'merge'
-            for key, value in results.items():
-                old_value = task_vars.get(key, None)
-                results[key] = combine_vars(old_value, value, merge=merge_hashes)
+            existing_variables = {k: v for k, v in task_vars.items() if k in results}
+            results = combine_vars(existing_variables, results, merge=merge_hashes)
 
         result['ansible_included_var_files'] = self.included_files
         result['ansible_facts'] = results
@@ -169,14 +167,14 @@ class ActionModule(ActionBase):
                 )
                 self.source_dir = path_to_use
         else:
-            if hasattr(self._task._ds, '_data_source'):
+            if (origin := self._task._origin) and origin.path:  # origin.path is not present for ad-hoc tasks
                 current_dir = (
-                    "/".join(self._task._ds._data_source.split('/')[:-1])
+                    "/".join(origin.path.split('/')[:-1])
                 )
                 self.source_dir = path.join(current_dir, self.source_dir)
 
     def _log_walk(self, error):
-        self._display.vvv('Issue with walking through "%s": %s' % (to_native(error.filename), to_native(error)))
+        self._display.vvv(f"Issue with walking through {error.filename}: {error}")
 
     def _traverse_dir_depth(self):
         """ Recursively iterate over a directory and sort the files in
@@ -205,9 +203,8 @@ class ActionModule(ActionBase):
             try:
                 if re.search(r'{0}$'.format(file_type), filename):
                     return True
-            except Exception:
-                err_msg = 'Invalid regular expression: {0}'.format(file_type)
-                raise AnsibleError(err_msg)
+            except Exception as ex:
+                raise AnsibleError(f'Invalid regular expression: {file_type!r}', obj=file_type) from ex
         return False
 
     def _is_valid_file_ext(self, source_file):
@@ -233,18 +230,18 @@ class ActionModule(ActionBase):
         err_msg = ''
         if validate_extensions and not self._is_valid_file_ext(filename):
             failed = True
-            err_msg = ('{0} does not have a valid extension: {1}'.format(to_native(filename), ', '.join(self.valid_extensions)))
+            err_msg = f"{filename!r} does not have a valid extension: {', '.join(self.valid_extensions)}"
         else:
-            b_data, show_content = self._loader._get_file_contents(filename)
-            data = to_text(b_data, errors='surrogate_or_strict')
+            data = self._loader.load_from_file(filename, cache='none', trusted_as_template=True)
 
-            self.show_content = show_content
-            data = self._loader.load(data, file_name=filename, show_content=show_content)
-            if not data:
+            self.show_content &= not SourceWasEncrypted.is_tagged_on(data)
+
+            if data is None:  # support empty files, but not falsey values
                 data = dict()
+
             if not isinstance(data, dict):
                 failed = True
-                err_msg = ('{0} must be stored as a dictionary/hash'.format(to_native(filename)))
+                err_msg = f"{filename!r} must be stored as a dictionary/hash"
             else:
                 self.included_files.append(filename)
                 results.update(data)
