@@ -1,4 +1,5 @@
 """Import the given python module(s) and report error(s) encountered."""
+
 from __future__ import annotations
 
 
@@ -43,6 +44,7 @@ def main():
         # noinspection PyCompatibility
         from importlib import import_module
     except ImportError:
+
         def import_module(name, package=None):  # type: (str, str | None) -> types.ModuleType
             assert package is None
             __import__(name)
@@ -60,11 +62,10 @@ def main():
 
     if collection_full_name:
         # allow importing code from collections when testing a collection
-        from ansible.module_utils.common.text.converters import to_bytes, to_text, to_native, text_type
 
         # noinspection PyProtectedMember
         from ansible.utils.collection_loader._collection_finder import _AnsibleCollectionFinder
-        from ansible.utils.collection_loader import _collection_finder
+        from ansible.utils.collection_loader import _collection_finder, _to_text
 
         yaml_to_dict_cache = {}
 
@@ -74,7 +75,7 @@ def main():
 
         def parse_value(value):
             """Custom value parser for JSON deserialization that recognizes our internal ISO date format."""
-            if isinstance(value, text_type):
+            if isinstance(value, str):
                 match = iso_date_re.search(value)
 
                 if match:
@@ -86,7 +87,7 @@ def main():
             """Object hook for custom ISO date deserialization from JSON."""
             return dict((key, parse_value(value)) for key, value in data.items())
 
-        def yaml_to_dict(yaml, content_id):
+        def yaml_to_dict(yaml: str | bytes, content_id):
             """
             Return a Python dict version of the provided YAML.
             Conversion is done in a subprocess since the current Python interpreter does not have access to PyYAML.
@@ -96,18 +97,18 @@ def main():
 
             try:
                 cmd = [external_python, yaml_to_json_path]
-                proc = subprocess.Popen([to_bytes(c) for c in cmd],  # pylint: disable=consider-using-with
-                                        stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                stdout_bytes, stderr_bytes = proc.communicate(to_bytes(yaml))
+                proc = subprocess.Popen(cmd,  # pylint: disable=consider-using-with
+                                        stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
+                stdout, stderr = proc.communicate(_to_text(yaml))
 
                 if proc.returncode != 0:
-                    raise Exception('command %s failed with return code %d: %s' % ([to_native(c) for c in cmd], proc.returncode, to_native(stderr_bytes)))
+                    raise Exception('command %s failed with return code %d: %s' % (cmd, proc.returncode, stderr))
 
-                data = yaml_to_dict_cache[content_id] = json.loads(to_text(stdout_bytes), object_hook=object_hook)
+                data = yaml_to_dict_cache[content_id] = json.loads(stdout, object_hook=object_hook)
 
                 return data
             except Exception as ex:
-                raise Exception('internal importer error - failed to parse yaml: %s' % to_native(ex))
+                raise Exception(f'internal importer error - failed to parse yaml: {ex}')
 
         _collection_finder._meta_yml_to_dict = yaml_to_dict  # pylint: disable=protected-access
 
@@ -142,11 +143,13 @@ def main():
 
     class ImporterAnsibleModule:
         """Replacement for AnsibleModule to support import testing."""
+
         def __init__(self, *args, **kwargs):
             raise ImporterAnsibleModuleException()
 
     class RestrictedModuleLoader:
         """Python module loader that restricts inappropriate imports."""
+
         def __init__(self, path, name, restrict_to_module_paths):
             self.path = path
             self.name = name
@@ -159,7 +162,7 @@ def main():
             loader = self._get_loader(fullname, path=path)
             if loader is not None:
                 if has_py3_loader:
-                    # loader is expected to be Optional[importlib.abc.Loader], but RestrictedModuleLoader does not inherit from importlib.abc.Loder
+                    # loader is expected to be Optional[importlib.abc.Loader], but RestrictedModuleLoader does not inherit from importlib.abc.Loader
                     return spec_from_loader(fullname, loader)  # type: ignore[arg-type]
                 raise ImportError("Failed to import '%s' due to a bug in ansible-test. Check importlib imports for typos." % fullname)
             return None
@@ -431,6 +434,7 @@ def main():
 
     class Capture:
         """Captured output and/or exception."""
+
         def __init__(self):
             # use buffered IO to simulate StringIO; allows Ansible's stream patching to behave without warnings
             self.stdout = TextIOWrapper(BytesIO())
@@ -489,6 +493,7 @@ def main():
         finally:
             if import_type == 'plugin' and not collection_loader:
                 from ansible.utils.collection_loader._collection_finder import _AnsibleCollectionFinder
+
                 _AnsibleCollectionFinder._remove()  # pylint: disable=protected-access
 
             if sys.meta_path[0] != restricted_loader:
@@ -541,12 +546,6 @@ def main():
                     "ignore",
                     "AnsibleCollectionFinder has already been configured")
 
-            # ansible.utils.unsafe_proxy attempts patching sys.intern generating a warning if it was already patched
-            warnings.filterwarnings(
-                "ignore",
-                "skipped sys.intern patch; appears to have already been patched"
-            )
-
             try:
                 yield
             finally:
